@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclEvent.c,v 1.4 1999/04/16 00:46:46 stanton Exp $
+ * RCS: @(#) $Id: tclEvent.c,v 1.5.6.1 1999/11/19 23:32:04 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -453,11 +453,12 @@ Tcl_DeleteExitHandler(proc, clientData)
 	    } else {
 		prevPtr->nextPtr = exitPtr->nextPtr;
 	    }
-	    Tcl_MutexUnlock(&exitMutex);
 	    ckfree((char *) exitPtr);
-	    return;
+	    break;
 	}
     }
+    Tcl_MutexUnlock(&exitMutex);
+    return;
 }
 
 /*
@@ -779,7 +780,7 @@ TclFindEncodings(argv0)
 	    TclpSetInitialEncodings();
 
 	    /*
-	     * Now convert the native sting back to native string back to UTF.
+	     * Now convert the native string back to UTF.
 	     */
 	     
 	    if (pathPtr != NULL) {
