@@ -3,7 +3,7 @@
 # Default system startup file for Tcl-based applications.  Defines
 # "unknown" procedure and auto-load facilities.
 #
-# RCS: @(#) $Id: init.tcl,v 1.29.2.1 1999/05/21 00:04:29 redman Exp $
+# RCS: @(#) $Id: init.tcl,v 1.32.4.1 1999/10/30 11:06:17 hobbs Exp $
 #
 # Copyright (c) 1991-1993 The Regents of the University of California.
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
@@ -16,7 +16,7 @@
 if {[info commands package] == ""} {
     error "version mismatch: library\nscripts expect Tcl version 7.5b1 or later but the loaded version is\nonly [info patchlevel]"
 }
-package require -exact Tcl 8.1
+package require -exact Tcl 8.2
 
 # Compute the auto path to use in this interpreter.
 # The values on the path come from several locations:
@@ -52,12 +52,10 @@ if {[string compare [info library] {}]} {
 	}
     }
 }
-foreach __dir [list [file dirname [info nameofexecutable]] \
-	[file join [file dirname [file dirname \
-	[info nameofexecutable]]] lib]] {
-    if {[lsearch -exact $auto_path $__dir] < 0} {
-	lappend auto_path $__dir
-    }
+set __dir [file join [file dirname \
+	[file dirname [info nameofexecutable]]] lib]
+if {[lsearch -exact $auto_path $__dir] < 0} {
+    lappend auto_path $__dir
 }
 if {[info exist tcl_pkgPath]} {
     foreach __dir $tcl_pkgPath {
@@ -223,7 +221,7 @@ proc unknown args {
 	    && [info exists tcl_interactive] && $tcl_interactive} {
 	if {![info exists auto_noexec]} {
 	    set new [auto_execok $name]
-	    if {$new != ""} {
+	    if {[string compare {} $new]} {
 		set errorCode $savedErrorCode
 		set errorInfo $savedErrorInfo
 		set redir ""
@@ -480,7 +478,7 @@ proc auto_import {pattern} {
 # Arguments: 
 # name -			Name of a command.
 
-if {[string compare $tcl_platform(platform) windows] == 0} {
+if {[string equal windows $tcl_platform(platform)]} {
 # Windows version.
 #
 # Note that info executable doesn't work under Windows, so we have to
@@ -522,14 +520,16 @@ proc auto_execok name {
 	append path "$windir/system;$windir;"
     }
 
-    if {[info exists env(PATH)]} {
-	append path $env(PATH)
+    foreach var {PATH Path path} {
+	if {[info exists env($var)]} {
+	    append path ";$env($var)"
+	}
     }
 
     foreach dir [split $path {;}] {
-	if {$dir == ""} {
-	    set dir .
-	}
+	# Skip already checked directories
+	if {[info exists checked($dir)] || [string equal {} $dir]} { continue }
+	set checked($dir) {}
 	foreach ext {{} .com .exe .bat} {
 	    set file [file join $dir ${name}${ext}]
 	    if {[file exists $file] && ![file isdirectory $file]} {
