@@ -18,7 +18,7 @@
 #	its .o file placed before all others in the command; then
 #	"ld" is executed to bind the objects together.
 #
-# SCCS: @(#) ldAout.tcl 1.11 96/09/17 09:02:20
+# RCS: @(#) $Id: ldAout.tcl,v 1.3 1998/11/11 02:39:31 welch Exp $
 #
 # Copyright (c) 1995, by General Electric Company. All rights reserved.
 #
@@ -44,12 +44,10 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
   # function now accepts both 2 and 3 parameters.
 
   if {$shlib_suffix==""} {
-    set shlib_suffix $env(SHLIB_SUFFIX)
     set shlib_cflags $env(SHLIB_CFLAGS)
   } else {
     if {$shlib_cflags=="none"} {
       set shlib_cflags $shlib_suffix
-      set shlib_suffix [info sharedlibextension]
     }
   }
 
@@ -100,9 +98,9 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
     } elseif {![string compare $a -o]} {
       set minusO 1
     }
-    if [regexp {^-[lL]} $a] {
+    if {[regexp {^-[lL]} $a]} {
 	lappend libraries $a
-	if [regexp {^-L} $a] {
+	if {[regexp {^-L} $a]} {
 	    lappend libdirs [string range $a 2 end]
 	}
     } elseif {$seenDotO} {
@@ -118,10 +116,10 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
 
   set libs {}
   foreach lib $libraries {
-      if [regexp {^-l} $lib] {
+      if {[regexp {^-l} $lib]} {
 	  set lname [string range $lib 2 end]
 	  foreach dir $libdirs {
-	      if [file exists [file join $dir lib${lname}_G0.a]] {
+	      if {[file exists [file join $dir lib${lname}_G0.a]]} {
 		  set lname ${lname}_G0
 		  break
 	      }
@@ -139,16 +137,22 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
     error "-o option must be supplied to link a Tcl load module"
   }
   set m [file tail $outputFile]
-  set l [expr [string length $m] - [string length $shlib_suffix]]
-  if [string compare [string range $m $l end] $shlib_suffix] {
-    error "Output file does not appear to have a $shlib_suffix suffix"
+  if {[regexp {\.a$} $outputFile]} {
+    set shlib_suffix .a
+  } else {
+    set shlib_suffix ""
   }
-  set modName [string tolower [string range $m 0 [expr $l-1]]]
-  if [regexp {^lib} $modName] {
+  if {[regexp {\..*$} $outputFile match]} {
+    set l [expr {[string length $m] - [string length $match]}]
+  } else {
+    error "Output file does not appear to have a suffix"
+  }
+  set modName [string tolower [string range $m 0 [expr {$l-1}]]]
+  if {[regexp {^lib} $modName]} {
     set modName [string range $modName 3 end]
   }
-  if [regexp {[0-9\.]*(_g0)?$} $modName match] {
-    set modName [string range $modName 0 [expr [string length $modName]-[string length $match]-1]]
+  if {[regexp {[0-9\.]*(_g0)?$} $modName match]} {
+    set modName [string range $modName 0 [expr {[string length $modName]-[string length $match]-1}]]
   }
   set modName "[string toupper [string index $modName 0]][string range $modName 1 end]"
   
@@ -156,7 +160,7 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
 
   set f [open $nmCommand r]
   while {[gets $f l] >= 0} {
-    if [regexp {T[ 	]*_?([A-Z][a-z0-9_]*_(Safe)?Init(__FP10Tcl_Interp)?)$} $l trash symbol] {
+    if {[regexp {T[ 	]*_?([A-Z][a-z0-9_]*_(Safe)?Init(__FP10Tcl_Interp)?)$} $l trash symbol]} {
       if {![regexp {_?([A-Z][a-z0-9_]*_(Safe)?Init)} $symbol trash s]} {
 	set s $symbol
       }
@@ -211,9 +215,14 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
 
   # Now compose and execute the ld command that packages the module
 
-  set ldCommand ld
-  foreach item $head {
-    lappend ldCommand $item
+  if {$shlib_suffix == ".a"} {
+    set ldCommand "ar cr $outputFile"
+    regsub { -o} $tail {} tail
+  } else {
+    set ldCommand ld
+    foreach item $head {
+      lappend ldCommand $item
+    }
   }
   lappend ldCommand tcl$modName.o
   foreach item $tail {
@@ -221,6 +230,9 @@ proc tclLdAout {{cc {}} {shlib_suffix {}} {shlib_cflags none}} {
   }
   puts stderr $ldCommand
   eval exec $ldCommand
+  if {$shlib_suffix == ".a"} {
+    exec ranlib $outputFile
+  }
 
   # Clean up working files
 
